package net.minecraftforge.items;

import net.minecraft.entity.item.EntityItem;
import net.minecraft.entity.player.EntityPlayer;
import net.minecraft.item.ItemStack;
import net.minecraft.nbt.NBTTagCompound;
import net.minecraft.world.World;
import net.minecraftforge.items.wrapper.PlayerMainInvWrapper;

public class ItemHandlerHelper
{
    public static ItemStack insertItem(IItemHandler dest, ItemStack stack, boolean simulate)
    {
        if (dest == null || stack == null)
            return stack;

        for (int i = 0; i < dest.getSlots(); i++)
        {
            stack = dest.insertItem(i, stack, simulate);
            if (stack == null || stack.field_77994_a <= 0)
            {
                return null;
            }
        }

        return stack;
    }

    public static boolean canItemStacksStack(ItemStack a, ItemStack b)
    {
        if (a == null || !a.func_77969_a(b))
            return false;

        final NBTTagCompound aTag = a.func_77978_p();
        final NBTTagCompound bTag = b.func_77978_p();
        return (aTag != null || bTag == null) && (aTag == null || aTag.equals(bTag));
    }

    /**
     * A relaxed version of canItemStacksStack that stacks itemstacks with different metadata if they don't have subtypes.
     * This usually only applies when players pick up items.
     */
    public static boolean canItemStacksStackRelaxed(ItemStack a, ItemStack b)
    {
        if (a == null || b == null || a.func_77973_b() != b.func_77973_b())
            return false;

        if (!a.func_77985_e())
            return false;

        // Metadata value only matters when the item has subtypes
        // Vanilla stacks non-subtype items with different metadata together
        // e.g. a stick with metadata 0 and a stick with metadata 1 stack
        if (a.func_77981_g() && a.func_77960_j() != b.func_77960_j())
            return false;

        final NBTTagCompound aTag = a.func_77978_p();
        final NBTTagCompound bTag = b.func_77978_p();
        return (aTag != null || bTag == null) && (aTag == null || aTag.equals(bTag));
    }

    public static ItemStack copyStackWithSize(ItemStack itemStack, int size)
    {
        if (size == 0)
            return null;
        ItemStack copy = ItemStack.func_77944_b(itemStack);
        if (copy != null)
            copy.field_77994_a = size;
        return copy;
    }

    /**
     * Inserts the ItemStack into the inventory, filling up already present stacks first.
     * This is equivalent to the behaviour of a player picking up an item.
     * Note: This function stacks items without subtypes with different metadata together.
     */
    public static ItemStack insertItemStacked(IItemHandler inventory, ItemStack stack, boolean simulate)
    {
        if (inventory == null || stack == null)
            return stack;

        // not stackable -> just insert into a new slot
        if (!stack.func_77985_e())
        {
            return insertItem(inventory, stack, simulate);
        }

        int sizeInventory = inventory.getSlots();

        // go through the inventory and try to fill up already existing items
        for (int i = 0; i < sizeInventory; i++)
        {
            ItemStack slot = inventory.getStackInSlot(i);
            if (canItemStacksStackRelaxed(slot, stack))
            {
                stack = inventory.insertItem(i, stack, simulate);

                if (stack == null)
                {
                    break;
                }
            }
        }

        // insert remainder into empty slots
        if (stack != null)
        {
            // find empty slot
            for (int i = 0; i < sizeInventory; i++)
            {
                if (inventory.getStackInSlot(i) == null)
                {
                    stack = inventory.insertItem(i, stack, simulate);
                    if (stack == null)
                    {
                        break;
                    }
                }
            }
        }

        return stack;
    }

    /** giveItemToPlayer without preferred slot */
    public static void giveItemToPlayer(EntityPlayer player, ItemStack stack) {
        giveItemToPlayer(player, stack, -1);
    }

    /**
     * Inserts the given itemstack into the players inventory.
     * If the inventory can't hold it, the item will be dropped in the world at the players position.
     *
     * @param player The player to give the item to
     * @param stack  The itemstack to insert
     */
    public static void giveItemToPlayer(EntityPlayer player, ItemStack stack, int preferredSlot)
    {
        IItemHandler inventory = new PlayerMainInvWrapper(player.field_71071_by);
        World world = player.field_70170_p;

        // try adding it into the inventory
        ItemStack remainder = null;
        // insert into preferred slot first
        if(preferredSlot >= 0)
        {
            remainder = inventory.insertItem(preferredSlot, stack, false);
        }
        // then into the inventory in general
        if(remainder != null)
        {
            remainder = insertItemStacked(inventory, stack, false);
        }

        // play sound if something got picked up
        if (remainder == null || remainder.field_77994_a != stack.field_77994_a)
        {
            world.func_72908_a(player.field_70165_t, player.field_70163_u, player.field_70161_v,
                    "random.pop", 0.2F, ((world.field_73012_v.nextFloat() - world.field_73012_v.nextFloat()) * 0.7F + 1.0F) * 2.0F);
        }

        // drop remaining itemstack into the world
        if (remainder != null && !world.field_72995_K)
        {
            EntityItem entityitem = new EntityItem(world, player.field_70165_t, player.field_70163_u + 0.5, player.field_70161_v, stack);
            entityitem.func_174867_a(40);
            entityitem.field_70159_w = 0;
            entityitem.field_70179_y = 0;

            world.func_72838_d(entityitem);
        }
    }
}
